@@ -180,7 +180,7 @@ menustruct TextMenu[] = {
 	{"Encoding", submenu(FontEncodings)},
 	{"Insert", submenu(TextSpacing)},             
 	{"Justification (keypad)", submenu(Justifs)},
-	{"Parameterize", action(promtparam, NULL)},
+	{"Parameterize", action(promptparam, NULL)},
 	{"Unparameterize", action(startunparam, Number(P_SUBSTRING))},
 	{" ", noaction},
 	{"Make Label (t)", action(starttext, Number(NORMAL))},
