@@ -140,7 +140,7 @@ eparamptr make_new_eparam(char *key)
    newepp->next = NULL;
    newepp->key = (char *)malloc(1 + strlen(key));
    strcpy(newepp->key, key);
-   newepp->pointno = 0;
+   newepp->pdata.refkey = NULL;		/* equivalently, sets pointno=0 */
 
    return newepp;
 }
@@ -219,6 +219,15 @@ void free_element_param(genericptr thiselem, eparamptr thisepp)
 	    lastepp->next = epp->next;
 	 else
 	    thiselem->passed = epp->next;
+
+	 /* If object is an instance and the pdata record is not NULL,	*/
+	 /* then this is an indirect reference with the reference key	*/
+	 /* stored as an allocated string in pdata.refkey, which needs	*/
+	 /* to be free'd.						*/
+
+	 if ((thiselem->type == OBJECT) && (epp->pdata.refkey != NULL))
+	    free(epp->pdata.refkey);
+
 	 free(epp->key);
 	 free(epp);
 	 break;
@@ -263,7 +272,8 @@ void indicateparams(genericptr thiselem)
    if (thiselem != NULL) {
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) { 
 	 ops = match_param(topobject, epp->key);
-	 k = epp->pointno;
+	 if (ops == NULL) continue;	/* error condition */
+	 k = epp->pdata.pointno;
 	 switch(ops->which) {
 	    case P_POSITION: case P_POSITION_X: case P_POSITION_Y:
 	       switch(thiselem->type) {
@@ -320,6 +330,7 @@ void setparammarks(genericptr thiselem)
    if (thiselem != NULL) {
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 	 ops = match_param(topobject, epp->key);
+	 if (ops == NULL) continue;	/* error condition */
          Tcl_SetVar(xcinterp, (char *)paramvars[ops->which], "true",
 		TCL_NAMESPACE_ONLY);
       }
@@ -382,6 +393,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 {
    genericptr *eptr, thiselem;
    int i, j, k, ival;
+   char *key;
    float fval;
    oparamptr ops, ips;
    eparamptr epp;
@@ -396,7 +408,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
       if (thiselem->passed == NULL) continue;	/* Nothing to substitute */
 
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
-	 k = epp->pointno;
+
 	 ips = (pinst != NULL) ?  find_param(pinst, epp->key) : NULL;
 	 ops = match_param(thisobj, epp->key);
 
@@ -407,6 +419,46 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	 else if (ops == NULL)
 	    continue;
 
+	 if (thiselem->type == OBJECT) {
+	    key = epp->pdata.refkey;
+	    if (key != NULL) {
+	       objinstptr thisinst;
+	       oparamptr refop, newop;
+
+	       thisinst = TOOBJINST(eptr);
+	       /* Sanity checks: refkey must exist in object */
+	       refop = match_param(thisinst->thisobject, key);
+	       if (refop == NULL) {
+		  Fprintf("Error:  Reference key %s does not exist in object %s\n",
+				key, thisinst->thisobject->name);
+		  continue;
+	       }
+	       /* "type" and "which" records must match */
+	       else if ((ops->which != refop->which) || (ops->type != refop->type)) {
+		  Fprintf("Error:  Reference parameter %s is different type "
+				"than source parameter %s\n", key, refop->key);
+		  continue;
+	       }
+	       /* If an instance value already exists, remove it */
+	       newop = match_instance_param(thisinst, refop->key);
+	       if (newop != NULL)
+		  free_instance_param(thisinst, newop);
+	       
+	       /* Create a new instance parameter */
+	       newop = copyparameter((ips != NULL) ? ips : ops);
+	       newop->next = thisinst->params;
+	       thisinst->params = newop;
+
+	       /* Change the key from the parent to the child */
+	       if (strcmp(ops->key, refop->key)) {
+		  free(newop->key);
+		  newop->key = strdup(refop->key);
+	       }
+	       continue;
+	    }
+	 }
+
+	 k = epp->pdata.pointno;
          switch(ops->which) {
 	    case P_POSITION_X:
 	       retval = max(retval, 1);
@@ -624,160 +676,177 @@ void pwriteback(objinstptr thisinst)
    objinstptr pinst;
    eparamptr epp;
    oparamptr ops, ips;
-   int i, j, k, *ival;
-   float *fval;
+   int i, j, k, *destivalptr, found;
    XPoint *setpt;
+   Boolean changed, need_redraw = FALSE;
+   union {
+      int ival;
+      float fval;
+   } wtemp;
 
    pinst = (thisinst == areastruct.topinstance) ? areastruct.topinstance : thisinst;
    thisobj = (pinst == NULL) ? topobject : pinst->thisobject;
 
-   /* make sure that all instance values exist */
+   /* Make sure that all instance values exist */
    if (pinst != NULL) copyparams(pinst, pinst);
 
-   for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++) {
+   /* Because more than one element can point to the same parameter, we search	*/
+   /* through each (numerical) parameter declared in the object.  If any	*/
+   /* element has a different value, the parameter is changed to match.   This	*/
+   /* operates on the assumption that no more than one element will change the	*/
+   /* value of any one parameter on a single call to pwriteback().		*/
 
-      thiselem = *eptr;
-      if (thiselem->passed == NULL) continue;	/* Nothing to write back */
-
-      for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
-
-	 k = epp->pointno;
-
-         ips = (pinst != NULL) ? match_instance_param(pinst, epp->key) : NULL;
-         ops = match_param(thisobj, epp->key);
-
-	 ival = (ips != NULL) ? &ips->parameter.ivalue : &ops->parameter.ivalue;
-	 fval = (ips != NULL) ? &ips->parameter.fvalue : &ops->parameter.fvalue;
-
-         switch(ops->which) {
-	    case P_POSITION_X:
-	       switch(thiselem->type) {
-	          case OBJECT:
-		     *ival = TOOBJINST(&thiselem)->position.x;
-		     break;
-	          case LABEL:
-		     *ival = TOLABEL(&thiselem)->position.x;
-		     break;
-	          case POLYGON:
-	             setpt = TOPOLY(&thiselem)->points + k;
-		     *ival = setpt->x;
-		     break;
-	          case ARC:
-		     *ival = TOARC(&thiselem)->position.x;
-		     break;
-	          case SPLINE:
-		     *ival = TOSPLINE(&thiselem)->ctrl[k].x;
-		     break;
+   for (ops = thisobj->params; ops != NULL; ops = ops->next) {
+      if (ops->which == P_SUBSTRING) continue;	/* handle numerical params only */
+      found = 0;
+      changed = FALSE;
+      ips = (pinst != NULL) ? match_instance_param(pinst, ops->key) : NULL;
+      for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++) {
+         thiselem = *eptr;
+         if (thiselem->passed == NULL) continue;	/* Nothing to write back */
+         for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
+	    if (!strcmp(epp->key, ops->key)) {
+	       found++;
+	       k = epp->pdata.pointno;
+               switch(ops->which) {
+	          case P_POSITION_X:
+	             switch(thiselem->type) {
+	                case OBJECT:
+		           wtemp.ival = TOOBJINST(&thiselem)->position.x;
+		           break;
+	                case LABEL:
+		           wtemp.ival = TOLABEL(&thiselem)->position.x;
+		           break;
+	                case POLYGON:
+	                   setpt = TOPOLY(&thiselem)->points + k;
+		           wtemp.ival = setpt->x;
+		           break;
+	                case ARC:
+		           wtemp.ival = TOARC(&thiselem)->position.x;
+		           break;
+	                case SPLINE:
+		           wtemp.ival = TOSPLINE(&thiselem)->ctrl[k].x;
+		           break;
+	             }
+	             break;
+	          case P_POSITION_Y:
+	             switch(thiselem->type) {
+	                case OBJECT:
+		           wtemp.ival = TOOBJINST(&thiselem)->position.y;
+		           break;
+	                case LABEL:
+		           wtemp.ival = TOLABEL(&thiselem)->position.y;
+		           break;
+	                case POLYGON:
+	                   setpt = TOPOLY(&thiselem)->points + k;
+		           wtemp.ival = setpt->y;
+		           break;
+	                case ARC:
+		           wtemp.ival = TOARC(&thiselem)->position.y;
+		           break;
+	                case SPLINE:
+		           wtemp.ival = TOSPLINE(&thiselem)->ctrl[k].y;
+		           break;
+	             }
+	             break;
+	          case P_STYLE:
+	             switch(thiselem->type) {
+	                case POLYGON:
+		           wtemp.ival = TOPOLY(&thiselem)->style;
+		           break;
+	                case ARC:
+		           wtemp.ival = TOARC(&thiselem)->style;
+		           break;
+	                case SPLINE:
+		           wtemp.ival = TOSPLINE(&thiselem)->style;
+		           break;
+	                case PATH:
+		           wtemp.ival = TOPATH(&thiselem)->style;
+		           break;
+	             }
+	             break;
+	          case P_JUSTIFY:
+	             switch(thiselem->type) {
+	                case LABEL:
+		           wtemp.ival = TOLABEL(&thiselem)->justify;
+		           break;
+	             }
+	             break;
+	          case P_ANGLE1:
+	             switch(thiselem->type) {
+	                case ARC:
+		           wtemp.fval = TOARC(&thiselem)->angle1;
+		           break;
+	             }
+	             break;
+	          case P_ANGLE2:
+	             switch(thiselem->type) {
+	                case ARC:
+		           wtemp.fval = TOARC(&thiselem)->angle1;
+		           break;
+	             }
+	             break;
+	          case P_RADIUS:
+	             switch(thiselem->type) {
+	                case ARC:
+		           wtemp.ival = TOARC(&thiselem)->radius;
+		           break;
+	             }
+	             break;
+	          case P_MINOR_AXIS:
+	             switch(thiselem->type) {
+	                case ARC:
+		           wtemp.ival = TOARC(&thiselem)->yaxis;
+		           break;
+	             }
+	             break;
+	          case P_ROTATION:
+	             switch(thiselem->type) {
+	                case OBJECT:
+		           wtemp.ival = TOOBJINST(&thiselem)->rotation;
+		           break;
+	                case LABEL:
+		           wtemp.ival = TOLABEL(&thiselem)->rotation;
+		           break;
+	             }
+	             break;
+	          case P_SCALE:
+	             switch(thiselem->type) {
+	                case OBJECT:
+		           wtemp.fval = TOOBJINST(&thiselem)->scale;
+		           break;
+	                case LABEL:
+		           wtemp.fval = TOLABEL(&thiselem)->scale;
+		           break;
+	             }
+	             break;
+	          case P_LINEWIDTH:
+	             switch(thiselem->type) {
+	                case POLYGON:
+		           wtemp.fval = TOPOLY(&thiselem)->width;
+		           break;
+	                case ARC:
+		           wtemp.fval = TOARC(&thiselem)->width;
+		           break;
+	                case SPLINE:
+		           wtemp.fval = TOSPLINE(&thiselem)->width;
+		           break;
+	                case PATH:
+		           wtemp.fval = TOPATH(&thiselem)->width;
+		           break;
+	             }
+	             break;
+               }
+	       destivalptr = (ips != NULL) ? &ips->parameter.ivalue
+			: &ops->parameter.ivalue;
+	       if ((!changed) && (wtemp.ival != *destivalptr)) {
+		  *destivalptr = wtemp.ival;
+		  changed = TRUE;
 	       }
+	       else if (found > 1) need_redraw = TRUE;
 	       break;
-	    case P_POSITION_Y:
-	       switch(thiselem->type) {
-	          case OBJECT:
-		     *ival = TOOBJINST(&thiselem)->position.y;
-		     break;
-	          case LABEL:
-		     *ival = TOLABEL(&thiselem)->position.y;
-		     break;
-	          case POLYGON:
-	             setpt = TOPOLY(&thiselem)->points + k;
-		     *ival = setpt->y;
-		     break;
-	          case ARC:
-		     *ival = TOARC(&thiselem)->position.y;
-		     break;
-	          case SPLINE:
-		     *ival = TOSPLINE(&thiselem)->ctrl[k].y;
-		     break;
-	       }
-	       break;
-	    case P_STYLE:
-	       switch(thiselem->type) {
-	          case POLYGON:
-		     *ival = TOPOLY(&thiselem)->style;
-		     break;
-	          case ARC:
-		     *ival = TOARC(&thiselem)->style;
-		     break;
-	          case SPLINE:
-		     *ival = TOSPLINE(&thiselem)->style;
-		     break;
-	          case PATH:
-		     *ival = TOPATH(&thiselem)->style;
-		     break;
-	       }
-	       break;
-	    case P_JUSTIFY:
-	       switch(thiselem->type) {
-	          case LABEL:
-		     *ival = TOLABEL(&thiselem)->justify;
-		     break;
-	       }
-	       break;
-	    case P_ANGLE1:
-	       switch(thiselem->type) {
-	          case ARC:
-		     *fval = TOARC(&thiselem)->angle1;
-		     break;
-	       }
-	       break;
-	    case P_ANGLE2:
-	       switch(thiselem->type) {
-	          case ARC:
-		     *fval = TOARC(&thiselem)->angle1;
-		     break;
-	       }
-	       break;
-	    case P_RADIUS:
-	       switch(thiselem->type) {
-	          case ARC:
-		     *ival = TOARC(&thiselem)->radius;
-		     break;
-	       }
-	       break;
-	    case P_MINOR_AXIS:
-	       switch(thiselem->type) {
-	          case ARC:
-		     *ival = TOARC(&thiselem)->yaxis;
-		     break;
-	       }
-	       break;
-	    case P_ROTATION:
-	       switch(thiselem->type) {
-	          case OBJECT:
-		     *ival = TOOBJINST(&thiselem)->rotation;
-		     break;
-	          case LABEL:
-		     *ival = TOLABEL(&thiselem)->rotation;
-		     break;
-	       }
-	       break;
-	    case P_SCALE:
-	       switch(thiselem->type) {
-	          case OBJECT:
-		     *fval = TOOBJINST(&thiselem)->scale;
-		     break;
-	          case LABEL:
-		     *fval = TOLABEL(&thiselem)->scale;
-		     break;
-	       }
-	       break;
-	    case P_LINEWIDTH:
-	       switch(thiselem->type) {
-	          case POLYGON:
-		     *fval = TOPOLY(&thiselem)->width;
-		     break;
-	          case ARC:
-		     *fval = TOARC(&thiselem)->width;
-		     break;
-	          case SPLINE:
-		     *fval = TOSPLINE(&thiselem)->width;
-		     break;
-	          case PATH:
-		     *fval = TOPATH(&thiselem)->width;
-		     break;
-	       }
-	       break;
-         }
+	    }
+	 }
       }
    }
 
@@ -785,6 +854,13 @@ void pwriteback(objinstptr thisinst)
    /* get erased (so they won't be written to the output unnecessarily) */
 
    if (pinst != NULL) resolveparams(pinst);
+
+   /* Because more than one element may use the same parameter,		*/
+   /* pwriteback checks for cases in which a change in one element	*/
+   /* precipitates a change in another.  If so, force a redraw.		*/
+
+   if (need_redraw && (thisinst == areastruct.topinstance))
+      drawarea(NULL, NULL, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -886,6 +962,32 @@ void resolveparams(objinstptr thisinst)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Return a copy of the single parameter "cops"			*/
+/*--------------------------------------------------------------*/
+
+oparamptr copyparameter(oparamptr cops)
+{
+   oparamptr newops;
+
+   newops = make_new_parameter(cops->key);
+   newops->next = NULL;
+   newops->type = cops->type;
+   newops->which = cops->which;
+   switch(cops->type) {
+      case XC_STRING:
+	 newops->parameter.string = stringcopy(cops->parameter.string);
+	 break;
+      case XC_INT: case XC_FLOAT:
+	 newops->parameter.ivalue = cops->parameter.ivalue;
+	 break;
+      default:
+	 Fprintf(stderr, "Error:  bad parameter\n");
+	 break;
+   }
+   return newops;
+}
+
 /*------------------------------------------------------*/
 /* Fill any NULL instance parameters with the values	*/
 /* from the calling instance, or from the instance	*/
@@ -904,36 +1006,27 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
 
    for (cops = psource; cops != NULL; cops = cops->next) {
       if (match_instance_param(destinst, cops->key) == NULL) {
-	 newops = make_new_parameter(cops->key);
+         newops = copyparameter(cops);
 	 newops->next = destinst->params;
 	 destinst->params = newops;
-	 newops->type = cops->type;
-	 newops->which = cops->which;
-         switch(cops->type) {
-	    case XC_STRING:
-	       newops->parameter.string = stringcopy(cops->parameter.string);
-	       break;
-	    case XC_INT: case XC_FLOAT:
-	       newops->parameter.ivalue = cops->parameter.ivalue;
-	       break;
-	    default:
-	       Fprintf(stderr, "Error:  bad parameter\n");
-	       break;
-	 }
       }
    }
 }
 
 /*--------------------------------------------------------------*/
-/* make a numerical (integer or float) parameter		*/
+/* Make a numerical (integer or float) parameter.		*/
+/* If "key" is non-NULL, then the parameter key will be set	*/
+/* from this rather than from the list "param_keys".  If the	*/
+/* key is an existing key with the same type as "mode", then	*/
+/* the new parameter will be linked to the existing one.	*/
 /*--------------------------------------------------------------*/
 
-void makenumericalp(genericptr *gelem, u_int mode)
+void makenumericalp(genericptr *gelem, u_int mode, char *key)
 {
    oparamptr ops, newops;
    eparamptr epp;
    XPoint *pptr;
-   char new_key[7];
+   char new_key[7], *keyptr;
    int pidx;
 
    /* Numerical parameters have designated keys to avoid the necessity	*/
@@ -954,7 +1047,7 @@ void makenumericalp(genericptr *gelem, u_int mode)
       return;
    }
 
-   /* cannot form a parameter on a top-level page */
+   /* Cannot form a parameter on a top-level page */
    if (is_page(topobject) != -1) {
       Wprintf("Cannot form a parameter in a top-level page!");
       return;
@@ -972,149 +1065,188 @@ void makenumericalp(genericptr *gelem, u_int mode)
 
    /* Ensure that no two parameters have the same name! */
 
-   strcpy(new_key, param_keys[mode]);
-   pidx = 0;
-   while (check_param(topobject, new_key)) {
-      pidx++;
-      sprintf(new_key, "%s%d", param_keys[mode], pidx);
+   if (key) {
+      keyptr = key;
+   }
+   else {
+      strcpy(new_key, param_keys[mode]);
+      pidx = 0;
+      while (check_param(topobject, new_key)) {
+         pidx++;
+         sprintf(new_key, "%s%d", param_keys[mode], pidx);
+      }
+      keyptr = new_key;
    }
-
-   newops = make_new_parameter(new_key);
-   newops->next = topobject->params;
-   topobject->params = newops;
-   newops->type = XC_INT;		/* most commonly used value */
-   newops->which = (u_char)mode;	/* what kind of parameter */
 
    /* Add the parameter to the element's parameter list */
 
-   epp = make_new_eparam(new_key);
+   epp = make_new_eparam(keyptr);
    epp->next = (*gelem)->passed;
    (*gelem)->passed = epp;
 
-   switch((*gelem)->type) {
-      case LABEL:
-	 switch(mode) {
-	    case P_POSITION_X:
-	       newops->parameter.ivalue = (int)TOLABEL(gelem)->position.x;
-	       break;
-	    case P_POSITION_Y:
-	       newops->parameter.ivalue = (int)TOLABEL(gelem)->position.y;
-	       break;
-	    case P_JUSTIFY:
-	       newops->parameter.ivalue = (int)TOLABEL(gelem)->justify;
-	       break;
-	    case P_ROTATION:
-	       newops->parameter.ivalue = (int)TOLABEL(gelem)->rotation;
-	       break;
-	    case P_SCALE:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOLABEL(gelem)->scale;
-	       break;
-	 }
-	 break;
-      case ARC:
-	 switch(mode) {
-	    case P_POSITION_X:
-	       newops->parameter.ivalue = (int)TOARC(gelem)->position.x;
-	       break;
-	    case P_POSITION_Y:
-	       newops->parameter.ivalue = (int)TOARC(gelem)->position.y;
-	       break;
-	    case P_ANGLE1:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOARC(gelem)->angle1;
-	       break;
-	    case P_ANGLE2:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOARC(gelem)->angle2;
-	       break;
-	    case P_RADIUS:
-	       newops->parameter.ivalue = (int)TOARC(gelem)->radius;
-	       break;
-	    case P_MINOR_AXIS:
-	       newops->parameter.ivalue = (int)TOARC(gelem)->yaxis;
-	       break;
-	    case P_STYLE:
-	       newops->parameter.ivalue = (int)TOARC(gelem)->style;
-	       break;
-	    case P_LINEWIDTH:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOARC(gelem)->width;
-	       break;
-	 }
-	 break;
-      case OBJECT:
-	 switch(mode) {
-	    case P_POSITION_X:
-	       newops->parameter.ivalue = (int)TOOBJINST(gelem)->position.x;
-	       break;
-	    case P_POSITION_Y:
-	       newops->parameter.ivalue = (int)TOOBJINST(gelem)->position.y;
-	       break;
-	    case P_ROTATION:
-	       newops->parameter.ivalue = (int)TOOBJINST(gelem)->rotation;
-	       break;
-	    case P_SCALE:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOOBJINST(gelem)->scale;
-	       break;
-	 }
-	 break;
-      case POLYGON:
-	 switch(mode) {
-	    case P_POSITION_X:
-	       pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
-	       newops->parameter.ivalue = (int)pptr->x;
-	       epp->pointno = areastruct.editcycle;
-	       break;
-	    case P_POSITION_Y:
-	       pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
-	       newops->parameter.ivalue = (int)pptr->y;
-	       epp->pointno = areastruct.editcycle;
-	       break;
-	    case P_STYLE:
-	       newops->parameter.ivalue = (int)TOPOLY(gelem)->style;
-	       break;
-	    case P_LINEWIDTH:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOPOLY(gelem)->width;
-	       break;
-	 }
-	 break;
-      case SPLINE:
-	 switch(mode) {
-	    case P_POSITION_X:
-	       pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
-	       newops->parameter.ivalue = (int)pptr->x;
-	       epp->pointno = areastruct.editcycle;
-	       break;
-	    case P_POSITION_Y:
-	       pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
-	       newops->parameter.ivalue = (int)pptr->y;
-	       epp->pointno = areastruct.editcycle;
-	       break;
-	    case P_STYLE:
-	       newops->parameter.ivalue = (int)TOSPLINE(gelem)->style;
-	       break;
-	    case P_LINEWIDTH:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOSPLINE(gelem)->width;
-	       break;
-	 }
-	 break;
-      case PATH:
-	 switch(mode) {
-	    case P_STYLE:
-	       newops->parameter.ivalue = (int)TOPATH(gelem)->style;
-	       break;
-	    case P_LINEWIDTH:
-	       newops->type = XC_FLOAT;
-	       newops->parameter.fvalue = TOPATH(gelem)->width;
-	       break;
-	 }
-	 break;
+   /* If keyptr does not point to an existing parameter, then we need	*/
+   /* to create it in the object's parameter list and set the default	*/
+   /* value to the existing value of the element.			*/
+
+   ops = match_param(topobject, keyptr);
+   if (ops == NULL) {
+      newops = make_new_parameter(keyptr);
+      newops->next = topobject->params;
+      topobject->params = newops;
+      newops->type = XC_INT;		/* most commonly used value */
+      newops->which = (u_char)mode;	/* what kind of parameter */
+      incr_changes(topobject);
+   }
+   else {
+      if (ops->which != (u_char)mode) {
+	 free_element_param(*gelem, epp);
+	 Fprintf(stderr, "Error: Attempt to link a parameter to "
+			"a parameter of a different type\n");
+	 return;
+      }
+      if ((newops = match_instance_param(areastruct.topinstance, keyptr)) == NULL) {
+         newops = make_new_parameter(keyptr);
+         newops->next = areastruct.topinstance->params;
+         areastruct.topinstance->params = newops;
+         newops->type = ops->type;
+         newops->which = ops->which;
+      }
+      else {
+         /* If the parameter exists and the instance has a non-default	*/
+         /* value for it, we will not change the instance record.  If	*/
+         /* the element value is different, then it will change, so we	*/
+	 /* should redraw.						*/
+         drawarea(NULL, NULL, NULL);
+         newops = NULL;
+	 return;
+      }
+   }
+
+   if (newops) {
+      switch((*gelem)->type) {
+         case LABEL:
+	    switch(mode) {
+	       case P_POSITION_X:
+	          newops->parameter.ivalue = (int)TOLABEL(gelem)->position.x;
+	          break;
+	       case P_POSITION_Y:
+	          newops->parameter.ivalue = (int)TOLABEL(gelem)->position.y;
+	          break;
+	       case P_JUSTIFY:
+	          newops->parameter.ivalue = (int)TOLABEL(gelem)->justify;
+	          break;
+	       case P_ROTATION:
+	          newops->parameter.ivalue = (int)TOLABEL(gelem)->rotation;
+	          break;
+	       case P_SCALE:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOLABEL(gelem)->scale;
+	          break;
+	    }
+	    break;
+         case ARC:
+	    switch(mode) {
+	       case P_POSITION_X:
+	          newops->parameter.ivalue = (int)TOARC(gelem)->position.x;
+	          break;
+	       case P_POSITION_Y:
+	          newops->parameter.ivalue = (int)TOARC(gelem)->position.y;
+	          break;
+	       case P_ANGLE1:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOARC(gelem)->angle1;
+	          break;
+	       case P_ANGLE2:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOARC(gelem)->angle2;
+	          break;
+	       case P_RADIUS:
+	          newops->parameter.ivalue = (int)TOARC(gelem)->radius;
+	          break;
+	       case P_MINOR_AXIS:
+	          newops->parameter.ivalue = (int)TOARC(gelem)->yaxis;
+	          break;
+	       case P_STYLE:
+	          newops->parameter.ivalue = (int)TOARC(gelem)->style;
+	          break;
+	       case P_LINEWIDTH:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOARC(gelem)->width;
+	          break;
+	    }
+	    break;
+         case OBJECT:
+	    switch(mode) {
+	       case P_POSITION_X:
+	          newops->parameter.ivalue = (int)TOOBJINST(gelem)->position.x;
+	          break;
+	       case P_POSITION_Y:
+	          newops->parameter.ivalue = (int)TOOBJINST(gelem)->position.y;
+	          break;
+	       case P_ROTATION:
+	          newops->parameter.ivalue = (int)TOOBJINST(gelem)->rotation;
+	          break;
+	       case P_SCALE:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOOBJINST(gelem)->scale;
+	          break;
+	    }
+	    break;
+         case POLYGON:
+	    switch(mode) {
+	       case P_POSITION_X:
+	          pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
+	          newops->parameter.ivalue = (int)pptr->x;
+	          epp->pdata.pointno = areastruct.editcycle;
+	          break;
+	       case P_POSITION_Y:
+	          pptr = TOPOLY(gelem)->points + areastruct.editcycle; 
+	          newops->parameter.ivalue = (int)pptr->y;
+	          epp->pdata.pointno = areastruct.editcycle;
+	          break;
+	       case P_STYLE:
+	          newops->parameter.ivalue = (int)TOPOLY(gelem)->style;
+	          break;
+	       case P_LINEWIDTH:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOPOLY(gelem)->width;
+	          break;
+	    }
+	    break;
+         case SPLINE:
+	    switch(mode) {
+	       case P_POSITION_X:
+	          pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
+	          newops->parameter.ivalue = (int)pptr->x;
+	          epp->pdata.pointno = areastruct.editcycle;
+	          break;
+	       case P_POSITION_Y:
+	          pptr = TOSPLINE(gelem)->ctrl + areastruct.editcycle; 
+	          newops->parameter.ivalue = (int)pptr->y;
+	          epp->pdata.pointno = areastruct.editcycle;
+	          break;
+	       case P_STYLE:
+	          newops->parameter.ivalue = (int)TOSPLINE(gelem)->style;
+	          break;
+	       case P_LINEWIDTH:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOSPLINE(gelem)->width;
+	          break;
+	    }
+	    break;
+         case PATH:
+	    switch(mode) {
+	       case P_STYLE:
+	          newops->parameter.ivalue = (int)TOPATH(gelem)->style;
+	          break;
+	       case P_LINEWIDTH:
+	          newops->type = XC_FLOAT;
+	          newops->parameter.fvalue = TOPATH(gelem)->width;
+	          break;
+	    }
+	    break;
+      }
    }
-   incr_changes(topobject);
 }
 
 /*--------------------------------------------------------------*/
@@ -1609,11 +1741,11 @@ void parameterize(int mode, char *key)
          makeparam(settext, key);
       }
       else if (mode == P_POSITION) {
-	 makenumericalp(topobject->plist + (*fselect), P_POSITION_X);
-	 makenumericalp(topobject->plist + (*fselect), P_POSITION_Y);
+	 makenumericalp(topobject->plist + (*fselect), P_POSITION_X, key);
+	 makenumericalp(topobject->plist + (*fselect), P_POSITION_Y, key);
       }
       else
-	 makenumericalp(topobject->plist + (*fselect), mode);
+	 makenumericalp(topobject->plist + (*fselect), mode, key);
    }
    objectdeselect();
    setparammarks(NULL);
